extends RigidBody

signal dies

export(float) var acceleration = 10
export(float) var maxSpeed = 10

export(float) var deathHeight = 5
var lastContactHeight = 0

onready var respawnTransform = get_transform()
onready var animPlayer = get_node("AnimationPlayer")

func _ready():
	# activate the _process()-method
	set_process(true)

# this is called every frame
func _process(delta):
	# check whether the player has died
	checkDeath()

func _integrate_forces(state):
	# if we don't touch anything controls are deactivated, so no steering in the air
	if state.get_contact_count() == 0:
		return
	
	# remember the y-value of the player's position when we are touching something so we can tell when the player is dead
	lastContactHeight = get_global_transform().origin.y
	
	# move sideways
	var movementDir = Vector3(0,0,0)
	if Input.is_action_pressed("player_down"):
		movementDir.z += 1
	if Input.is_action_pressed("player_up"):
		movementDir.z -= 1
	if Input.is_action_pressed("player_right"):
		movementDir.x += 1
	if Input.is_action_pressed("player_left"):
		movementDir.x -= 1
	
	state.add_force(acceleration * movementDir, Vector3(0,0,0))
	
	# limit the speed to maxSpeed
	var linearSpeed = state.get_linear_velocity()
	if linearSpeed.length() > maxSpeed:
		linearSpeed = linearSpeed.normalized() * maxSpeed
		state.set_linear_velocity(linearSpeed)

func checkDeath():
	# Compare the height of the position where the ground was touched last
	# with the height of the current position.
	# If the distance is too great the player is considered dead.
	# This method has the advantage that you can still build vertical levels
	# and the player doesn't have to drop all the way down before he is respawned.
	if lastContactHeight - get_global_transform().origin.y > deathHeight:
		emit_signal("dies")

func respawn():
	# Only play the respawn animation if it is not already currently running.
	if not animPlayer.is_playing() or animPlayer.get_current_animation() != "respawn":
		# Let the respawn animation handle the respawning.
		animPlayer.play("respawn")

# This is called by a Call Func Track in the respawn animation.
func reset():
	# Reset the position and stop any current movement.
	set_transform(respawnTransform)
	set_linear_velocity(Vector3(0,0,0))
	set_angular_velocity(Vector3(0,0,0))

func collect(collectable):
	# This would be the place to react to collectables, e.g. increase the player's score
	pass
